<?php
/*
OekakiPoteto 5.x Copyright 2000-2002 RanmaGuy (Theo Chakkapark, http://suteki.nu) and Marcello Basta-Forte (http://marcello.cellosoft.com). Modification to the files are permitted as long as my name remains on the files.

Wacintaki Poteto modifications Copyright 2004-2007 Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 1.3.10 - Last modified 9/23/2007
*/


// Load bootstrap
define ('BOOT', 1);


// Forces file to be writable
// Returns FALSE bad input or missing file, 0 failure, 1 success
// INPUT: $filename, int CHMOD
function check_write($in, $mod = 755) {
	if ($mod == 0 || empty($in)) {
		// Bad input
		return FALSE;
	}
	$mod = sprintf("%04d", $mod);

	if (!file_exists($in)) {
		return FALSE;
	}

	if (!is_writable ($in)) {
		if (! @chmod($in, $mod)) {
			return 0;
		}
	}
	return 1;
}


// get_dir_list: collects all files w/specific file extension(s)
// returns array(), FALSE
function get_dir_list($dir, $extensions = '') {
	$files = array();

	// PHP5 only
	if (empty ($extensions) && function_exists ('scandir')) {
		return scandir ($dir);
	}

	$dh = opendir ($dir);
	if (!$dh) return FALSE;

	while (FALSE !== ($filename = readdir ($dh))) {
		if (empty ($extensions)) {
			$files[] = $filename;
		} else {
			foreach ($extensions as $ext) {
				if (strpos ($filename, '.'.$ext) !== FALSE) {
					$files[] = $filename;
				}
			}
		}
	}

	closedir ($dh);
	return $files;
}


// Config
$CFG_update['debug']   = 0; // Prevents deletion of work files
$CFG_update['status']  = 0; // Returned by functions (0=STOP, >0=SALVAGE)
$CFG_update['version'] = '1.3.0'; // Update to this version

// Admin rank numbers
define ('RANK_MOD',    4);
define ('RANK_ADMIN',  5);
define ('RANK_SADMIN', 7);
define ('RANK_OWNER',  9);


// Init
error_reporting (E_ALL ^ E_NOTICE);

include ('common.php');
print_header();


// Import old config files
{
	$imports = array('config.php', 'dbconn.php');
	$import_fail = 0;
	$err_write = array();
	$err_include = array();
	foreach ($imports as $file) {
		if (file_exists ($file) && (! is_writable($file)) ) {
			$import_fail = 1;
			$err_write[] = $file; // push_array() shortcut
		}
		if (! @include ($file)) {
			$import_fail = 1;
			$err_include[] = $file;
		}
	}
	if ($import_fail > 0) {
echo <<<EOF

<hr />

<p>
	<strong>
		Warning:  update failed due to the following errors:<br />
		<p>

EOF;

		// Not writable
		if (count($err_write) > 0) {
			echo "			Files not writable:  ";
			web_line( make_list($err_write, ', ', '"'));
		}

		// Cannot include
		if (count($err_include) > 0) {
			echo "			Files cannot be included:  ";
			web_line( make_list($err_include, ', ', '"'));
		}

echo <<<EOF
		</p>
	</strong>
	Update cannot continue.<br />
	<br />
	Make sure your old board is working properly before running this updater.
</p>
EOF;

		print_footer();
		exit;
	}
}


/* Get the current version numbers
	$old_versions = Array, for detecting bad/incomplete update
	$from_version_split = Array, seperated numbers for existing board
	$to_version_split   = Array, seperated numbers for new version
*/
$old_versions = array('cfg' => '', 'db' => '');
$from_version_split = array();
$to_version_split = array();
{
	// Poteto 4.x
	if (empty ($jpgcompression) && empty ($CFG_version)) {
		// (Assumed pre-5.0 release)
		$old_versions['cfg'] = 'op-4.0.0';
	}
	// Poteto 5.x
	if (isset ($jpgcompression) && empty ($CFG_version)) {
		$old_versions['cfg'] = 'op-5.0.0';
	}
	// Wac1.0
	if (isset ($requireart) && empty ($CFG_version)) {
		$old_versions['cfg'] = 'wac-1.0.0';
	}
	// wac-1.1.0 - 1.1.5
	if (isset ($CFG_version)) {
		$old_versions['cfg'] = $CFG_version;

		$result = mysql_query ('SELECT miscstring FROM '.$OekakiPoteto_Prefix."oekakimisc WHERE miscname='dbversion'");
		if ($result) {
			$old_versions['db'] = mysql_result ($result, 0);
		}
	}
	// wac-1.2.0 +
	if (isset ($cfg['version'])) {
		$old_versions['cfg'] = $cfg['version'];

		$result = mysql_query ('SELECT miscstring FROM '.$OekakiPoteto_Prefix."oekakimisc WHERE miscname='dbversion'");
		if ($result) {
			$old_versions['db'] = mysql_result ($result, 0);
		}
	}

	if (isset ($_POST['from_version'])) {
		$from_version_split = version_split($_POST['from_version']);
		$to_version_split = version_split($_POST['to_version']);
	}
}


// Print update options
if (!isset ($_POST['stage'])) {
	print_update_type($old_versions['cfg']);
}


// Collect update options; print control panel options
if (w_gpc('stage') == '1') {
	print_control_panel();
}


// Collect control panel options; perform the update
if (w_gpc('stage') == '2') {

	echo "<p>\n";

	// OekakiPoteto update only
	if ($_POST['from_version'] == 'op-4.0.0' && $_POST['to_version'] == 'op-5.0.0') {
		if (op400_to_op500()) {
			print_OK();
		} else {
			print_verify();
		}
	}

	// Main Wacintaki update
	// Choose a starting point, and drop through all cases (no breaks)

	// Error message cascaded to next case if there's an error
	$next_error = '';

	switch ($_POST['from_version']) {
		case 'op-4.0.0':
			if (! op400_to_op500() ) {
				print_verify($next_error);
				break;
			}
			$next_error = 'Update to OP 5.x successful, but update to Wacintaki 1.0.0 failed.';
		case 'op-5.0.0':
			if (! op500_to_wac100() ) {
				print_verify($next_error);
				break;
			}
		case 'wac-1.0.0':
			if (! wac100_to_wac110(false) ) {
				print_verify('Update to Wacintaki 1.1.0 failed.');
				break;
			}
		case 'wac-1.1.0':
			if (! wac110_to_wac111() ) {
				print_verify('Update to Wacintaki 1.1.1 failed.');
				break;
			}
		case 'wac-1.1.1':
			if (! wac111_to_wac120() ) {
				print_verify('Update to Wacintaki 1.2.0 failed.');
				break;
			}
		case '1.2.0':
			if (! wac120_to_wac130() ) {
				print_verify('Update to Wacintaki 1.3.0 failed.');
				break;
			}
		case '5.6.0':
			if (! wax560_to_wac130() ) {
				print_verify('Update to Wacintaki 1.3.0 failed.');
				break;
			}

			// Latest version.  Dropout
			print_OK();
			break;
		case $CFG_update['version']:
			if (! wac120_to_wac130() ) {
				print_verify('Verification of '.$CFG_update['version'].' failed.');
			}

			// Latest version.  Dropout
			print_OK();
			break;
		default:
echo <<<EOF
	<strong>Unsupported update type &ldquo;From {$_POST['from_version']} to {$_POST['to_version']}&rdquo; using updater version &ldquo;{$CFG_update['version']}&rdquo;!</strong><br />
	Board config version:  &ldquo;{$old_versions['cfg']}&rdquo;,  database version:  &ldquo;{$old_versions['db']}&rdquo;<br />
	<br />
	Update cannot continue.<br />
	<br />
	If problems persist, try contacting me or visiting a Wacintaki support board and report the version information above.  A list of contacts is in the readme file that came with this updater.<br />
EOF;
	} // End switch()

	echo "</p>\n";
} // End Stage 2


print_footer();


// Finis
exit;



// ==================================================================
// General Functions
//
function print_header() {
	global $CFG_update;

	header('Content-type: text/html');
echo <<<EOF
<html>
<head>
	<title>OekakiPoteto Update</title>

	<style type="text/css" title="default">
		<!--
		h1 {
			font-family: "Helvetica", "Verdanna", sans-serif;
		}
		p {
			margin-left: 2em;
		}
		-->
	</style>
</head>

<body>


<h1>Wacintaki Poteto Update {$CFG_update['version']}</h1>

<hr />

EOF;
}


function print_footer() {
	echo "\n\n</body>\n</html>";
}


function print_OK() {
	global $CFG_update;

echo <<<EOF
	<p>
		If you have multiple boards, make sure each board is at the current version.<br />
		<br />
		Click the button below to finalize the update.  This will delete the updater files and prevent security problems.<br />
	</p>

	<form action="index.php" method="post">

EOF;
	if ($CFG_update['debug'] == 0) {
		echo'		<input name="action" type="hidden" value="secure" />';
	}
echo <<<EOF
		<input name="submit" type="submit" value="Secure updater and go to the BBS" />
	</form>

EOF;
}


function print_verify($error) {
	global $CFG_update;

	if (!empty ($error)) {
		web_line('');
		web_line($error);
	}

	if ($CFG_update['status'] > 0) {
		// Warning returned
echo <<<EOF
	<p>
		Errors occured during the update!  Please check the causes of the errors and run the updater again to ensure everything is set properly.  The updater may be run multiple times if required.
	</p>
EOF;
	} else {
		// Critical error
echo <<<EOF
	<p>
		Check your server permissions, the <em>config.php</em> and <em>dbconn.php</em> files, and try again.
	</p>
EOF;
	}
}


function print_update_type($detected_version) {
	global $CFG_update;

	// OekakiPoteto update
	if (substr($detected_version, 0, 2) == 'op') {
echo <<<EOF
<p>
	<strong>NOTICE:</strong>  If updating to Wacintaki, make sure you've deleted your old OekakiPoteto v5.x template and language files before running this updater.  Your old OekakiPoteto templates and language files WILL NOT WORK with Wacintaki Poteto.  See the readme file for more information.<br />
	<br />
	If updating from OekakiPoteto v4.x to v5.x, simply run this updater.
</p>

<hr />


EOF;
	}


	// v4.x update
	if ($detected_version == 'op-4.0.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>OekakiPoteto v4.x suspected.  If this is correct, please select an update:</h4>
	<p>
		<input name="from_version" type="hidden" value="op-4.0.0" />
		<input name="to_version" type="radio" value="{$CFG_update['version']}" checked="checked" />From OP 4.x to Wacintaki {$CFG_update['version']}<br />
		<input name="to_version" type="radio" value="op-5.0.0" />From OP 4.x to OP 5.1.0a<br />
	</p>

	<p>
		If you are <strong>NOT</strong> using OekakiPoteto v4.x, please update to at least v4.x before running the Wacintaki updater or your BBS may be damaged!
	</p>

	<h4>Click Next to continue...</h4>
	<p>
		<input name="stage" type="hidden" value="1" />
		<input name="submit" type="submit" value="Next" />
	</p>
</form>

EOF;
	return 1;
	}


	// v5.x update
	if ($detected_version == 'op-5.0.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>OekakiPoteto v5.x detected.</h4>
	<p>
		<input name="from_version" type="hidden" value="op-5.0.0" />
		<input name="to_version" type="hidden" value="wac-1.1.1" />
		Update will be to Wacintaki {$CFG_update['version']}<br />
	</p>

EOF;
echo <<<EOF
	<h4>Click Next to continue...</h4>
	<p>
		<input name="stage" type="hidden" value="1" />
		<input name="submit" type="submit" value="Next" />
	</p>
</form>

EOF;
	return 1;
	}


	// v5.x update
	if ($detected_version == '5.6.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>Wax Poteto v5.6.x detected.</h4>
	<p>
		<input name="from_version" type="hidden" value="5.6.0" />
		<input name="to_version" type="hidden" value="1.3.0" />
		Update will be to Wacintaki {$CFG_update['version']}<br />
	</p>

EOF;
echo <<<EOF
	<h4>Click Next to continue...</h4>
	<p>
		<input name="stage" type="hidden" value="1" />
		<input name="submit" type="submit" value="Next" />
	</p>
</form>

EOF;
	return 1;
	}


	// Wacintaki update
	//
	// Make sure current BBS is not more recent than updater
	$detected_version_split = version_split($detected_version);
	if ($detected_version_split['ver'] < 1003000) {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>Wacintaki version "$detected_version" detected.</h4>
	<p>
		Update will be to Wacintaki {$CFG_update['version']}<br />
	</p>

	<h4>Click Next to continue...</h4>
	<p>
		<input name="from_version" type="hidden" value="$detected_version" />
		<input name="to_version" type="hidden" value="{$CFG_update['version']}" />

		<input name="stage" type="hidden" value="1" />
		<input name="submit" type="submit" value="Next" />
	</p>
</form>

EOF;
		return 1;
	}

	if ($detected_version == $CFG_update['version']) {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>Wacintaki version "$detected_version" detected.</h4>
	<p>
		You appear to be running the latest version of Wacintaki already.  You may proceed to verify that the last update completed correctly.<br />
	</p>

	<h4>Click Next to continue...</h4>
	<p>
		<input name="from_version" type="hidden" value="$detected_version" />
		<input name="to_version" type="hidden" value="{$CFG_update['version']}" />

		<input name="stage" type="hidden" value="1" />
		<input name="submit" type="submit" value="Next" />
	</p>
</form>

EOF;
		return 1;
	}

	// Unknown version
echo <<<EOF
	<h4>Wacintaki version "$detected_version" detected.</h4>
	<p>
		This updater only supports Wacintaki versions less than or equal to {$CFG_update['version']}.<br />
		<br />
		Update cannot continue.
	</p>

EOF;
	return 0;
}


function print_control_panel() {
	global $CFG_update;

	// OekakiPoteto 4.x to 5.1.0a
	if ($_POST['from_version'] == 'op-4.0.0' && $_POST['to_version'] == 'op-5.0.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>No more information required.  Update to OekakiPoteto v5.1.0a...</h4>
	<p>
		<input name="from_version" type="hidden" value="op-4.0.0" />
		<input name="to_version" type="hidden" value="op-5.0.0" />

		<input name="stage" type="hidden" value="2" />
		<input name="submit" type="submit" value="Update" />
	</p>
</form>

EOF;
	} elseif ($_POST['from_version'] == 'op-4.0.0' || $_POST['from_version'] == 'op-5.0.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>Control Panel:</h4>
	<p>
		Additional information is required for the update.<br />
		<br />
		Note: all options will be set to their default values.  You should verify everything in the Contol Panel after the update has completed.
	</p>

	<h4>Select a new default template.</h4>
	<p>
		<select name="updatetemplate">

EOF;

		if ($handle = opendir('templates')) {
			/* This is the correct way to loop over the directory. */
			while (false !== ($file = readdir($handle))) {
				if(substr($file,-4) == '.php'){
					if ($file == 'Banana.php') {
?>
			<option value="<?=substr($file,0,-4)?>" selected="selected"><?=substr($file,0,-4);?></option>
<?php
					} else {
?>
			<option value="<?=substr($file,0,-4)?>"><?=substr($file,0,-4);?></option>
<?php
					}
				}
			}
			closedir($handle);
		}

echo <<<EOF
		</select>
	</p>

	<h4>Enter your name as the BBS Author.</h4>
	<p>
		<input name="BBSauthor" type="text" value="" />
	</p>

	<h4>Is this strictly an adult-rated BBS?</h4>
	<p>
		<select name="BBSadult">
			<option value="yes">Yes</option>
			<option value="no" selected="selected">No</option>
		</select>
	</p>

	<h4>Begin the update...</h4>
	<p>
		<input name="from_version" type="hidden" value="{$_POST['from_version']}" />
		<input name="to_version" type="hidden" value="{$_POST['to_version']}" />

		<input name="stage" type="hidden" value="2" />
		<input name="submit" type="submit" value="Update" />
	</p>
</form>
EOF;
	} elseif ($_POST['from_version'] == 'wac-1.1.0') {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>Control Panel:</h4>
	<p>
		Additional information is required for the update.<br />
		<br />
		All members may now be granted access to the upload feature, though by default only admins may upload pictures.  Use the control panel to enable/disable automatic assignment of the upload flag when people register, and limit upload sizes.
	</p>

	<h4>Do you want to allow new users to automatically get the upload flag?</h4>
	<p>
		<select name="upload_flag">
			<option value="no" selected="selected">No, I'll manually choose who may upload stuff</option>
			<option value="yes">Yes, anyone may upload pictures less than the max allowed size</option>
		</select>
	</p>

	<h4>Do you want to give upload access to all currently registered members with a draw flag?</h4>
	<p>
		<select name="upload_grant">
			<option value="no" selected="selected">No, let me manually approve upload flags</option>
			<option value="strip">No, strip every non-admin of their upload flags</option>
			<option value="yes">Yes, give upload access to all registered artists</option>
		</select>
	</p>
	<p>
		<input name="from_version" type="hidden" value="{$_POST['from_version']}" />
		<input name="to_version" type="hidden" value="{$_POST['to_version']}" />

		<input name="stage" type="hidden" value="2" />
		<input name="submit" type="submit" value="Update" />
	</p>
</form>

EOF;
	} else {
echo <<<EOF
<form name="form1" action="update.php" method="post">
	<h4>All new features will be set to their defaults.  View the Control Panel after the update to configure the new options.</h4>
	<p>
		<input name="from_version" type="hidden" value="{$_POST['from_version']}" />
		<input name="to_version" type="hidden" value="{$_POST['to_version']}" />

		<input name="stage" type="hidden" value="2" />
		<input name="submit" type="submit" value="Update" />
	</p>
</form>

EOF;
	}
}


// ==================================================================
// Update Functions
//
function op400_to_op500() {
	global $CFG_update;

	// Poteto v4.x to Poteto v5.1.0a
	$newversion = '5.1.0a';

	web_line('Update from v4.x to v5.1.0a...');

	/* We need to do the following:
		1) Add new values to config file:
			(UNKNOWN)

		2) Remove values from config file:
			(UNKNOWN)

		3) Add values to database:
			TABLE: oekakimailbox
			TABLE: oekakimisc
			TABLE: oekakichatmod
			TABLE: oekakichatmodusers

		4) Remove files if they exist:
			count
			install.php
	*/


	// Include config files
	if ( (! @include_once('dbconn.php')) && (! @include_once('config.php')) ) {
		web_line('<strong>STOP</strong>:  Cannot open "dbconn.php" and "config.php" files!');

		// Nothing done yet, so we can bail safely
		return 0;
	}


	// Upgrade count
	$resno = 0;
	if (file_exists ('count')) {
		$count_filesize = filesize ('count');
		$fp2 = @fopen ('count', 'r');
		if ($fp2) {
			$resno = (int) fread ($fp2, $count_filesize);
			fclose ($fp2);
		} else {
			web_line('<strong>STOP</strong>:  Cannot get picture count from "count" file!');

			// Nothing done yet, so we can bail safely
			return 0;
		}
	}

	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakichat RENAME ".$OekakiPoteto_MemberPrefix."oekakichat");
	sql_check($result);
	@$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakionline RENAME ".$OekakiPoteto_MemberPrefix."oekakionline");
	sql_check($result);
	@$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakikill RENAME ".$OekakiPoteto_MemberPrefix."oekakikill");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekakionline ADD onlineboard text");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki CHANGE commcount commcount INT(11) DEFAULT 0 NOT NULL");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki CHANGE piccount piccount INT(11) DEFAULT 0 NOT NULL");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekakionline CHANGE OID OID BIGINT(255) NOT NULL AUTO_INCREMENT");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekakionline CHANGE locale locale VARCHAR(30) DEFAULT NULL");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakidta CHANGE PIC_ID PIC_ID int(11) DEFAULT 0 NOT NULL ");
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakidta CHANGE edittime edittime INT(20) UNSIGNED DEFAULT 0");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakidta ADD postlock tinyint(4) NOT NULL default 0 ");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki ADD age text");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_Prefix."oekakidta ADD animation tinyint(2) default 0"); 
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki ADD gender text");
	sql_check($result);
	$result = mysql_query ("UPDATE ".$OekakiPoteto_Prefix."oekakidta SET postlock='1'");
	sql_check($result);
	$result = mysql_query ("CREATE TABLE ".$OekakiPoteto_MemberPrefix."oekakimailbox (MID int(11) NOT NULL auto_increment, sender varchar(255) NOT NULL DEFAULT '', reciever varchar(255) NOT NULL default '', subject text, body longtext,  mstatus smallint(10) NOT NULL default 0, senddate datetime NOT NULL default '0000-00-00 00:00:00', PRIMARY KEY (MID), KEY sender(sender), KEY reciever (reciever))"); 
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki ADD language text");
	sql_check($result);
	$result = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET language='english'");
	sql_check($result);
	$result = mysql_query ("CREATE TABLE ".$OekakiPoteto_Prefix."oekakimisc (MiscID int(11) NOT NULL auto_increment, miscname varchar(255) NOT NULL default '', miscvalue bigint(255) NOT NULL, PRIMARY KEY (MiscID),  KEY miscname (miscname))");
	sql_check($result);


	$result = mysql_query ("INSERT INTO ".$OekakiPoteto_Prefix."oekakimisc VALUES (1, 'piccount', $resno)");
	sql_check($result);
	$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki ADD picview TINYINT(11) DEFAULT 0 NOT NULL");
	sql_check($result);


	// Update chat
	$result = mysql_query ("CREATE TABLE ".$OekakiPoteto_MemberPrefix."oekakichatmod (usrname varchar(255) NOT NULL default '', command varchar(32) NOT NULL default 'MESSAGE',  comment tinytext, posttime timestamp(14) NOT NULL,  hostname text, email varchar(255) default NULL, url text, IP text, postname varchar(50) NOT NULL default '',  PRIMARY KEY (posttime)) TYPE=MyISAM;");
	sql_check($result);

	$result = mysql_query ("CREATE TABLE ".$OekakiPoteto_MemberPrefix."oekakichatmodusers (usrname varchar(255) NOT NULL, lastmessage timestamp(14) NOT NULL, afk ENUM('no','yes') NOT NULL default 'no', PRIMARY KEY (usrname)) TYPE=MyISAM;");
	sql_check($result);

	mysql_close ($dbconn);


	// Upgrade config
	// chmod('config.php', 0666);
	// chmod('dbconn.php', 0666);
	$fp = @fopen('config.php', 'w');
	if ($fp) {
		fwrite($fp, "<?php\n\n \$saltenc=\"".$saltenc."\"; \n \$OPpics=\"".$OPpics."\"; \n \$killuser=\"".$killuser."\"; \n \$template=\"".$template."\"; \n \$BBStitle=\"".$BBStitle."\"; \n \$eaddr=\"".$eaddr."\"; \n \$okurl=\"".$okurl."\"; \n \$apostrophes=\"".$apostrophes."\"; \n \$pstore=\"".$pstore."\"; \n \$ppage=\"".$ppage."\"; \n \$approval=\"".$approval."\"; \n \$drawaccess=\"".$drawaccess."\"; \n \$animationaccess=\"".$animationaccess."\"; \n \$xCanvas=\"".$xCanvas."\"; \n \$yCanvas=\"".$yCanvas."\"; \n \$xDefault=\"".$xDefault."\"; \n \$yDefault=\"".$yDefault."\"; \n \$chatMax=\"".$chatMax."\"; \n \$chatDisp=\"".$chatDisp."\"; \n \$language=\""."english"."\"; \n \$archivedir=\"".$archivedir."\"; \n \$guestpost=\"".$guestpost."\";\n \$anisize=\"".$anisize."\"; \n \$jpgcompression=\""."no"."\"; \n \$jpgcompressqual=\""."100"."\";\n\n?>");
		fclose($fp);
	} else {
		web_line('ERROR: Could not create new <em>config.php</em> file!');
		$CFG_update['status'] = 1;
	}


	// Remove old count file and install.php if they exist
	@unlink('count');
	if ($CFG_update['debug'] == 0) @unlink('install.php');


	//command to attempt to chmod the picture directory
	if (!check_write($OPpics, 777)) {
		web_line('ERROR: Picture directory "'.$OPpics.'" is not writable!  CHMOD the folder to 777 after the updater has finished.');
		$CFG_update['status'] = 1;
	}

	// Complete!
	if ($CFG_update['status'] == 0) {
		web_line('Update finished: OP v5.1.0a');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END op400_to_op500



function op500_to_wac100() {
	global $CFG_update;

	// Poteto v5.1.0a to Wacintaki Poteto v1.0.0
	$newversion = 'wac-1.0.0';

	web_line('Update from v5.1.0a to Wacintaki Poteto v1.0.0...');

	/* We need to do the following:
		1) Add new values to config file:
			$BBSauthor        = '';
			$OPPrefix         = '';
			$requireart       = 'no';
			$templatepath     = 'templates';
			$resourcepath     = 'resource';
			$perpage          = '25';
			$CFG_previewImg   = 'preview.png';
			$CFG_previewTitle = 'Canvas Preview';
			$xMin             = '50';
			$yMin             = '50';
			$aCanvas          = '250000'; (Unused for the moment)
			$useChat          = 'yes'; (Default for Poteto 5.1.0a)

		2) Remove values from config file:
			$apostrophes

		3) Update templates
			Ensure default is set, and change all user accounts to the default template.

		4) Verify that the chosen language file is available in Wacintaki Poteto.

		5) Remove files if they exist:
			announce.php
			arecover.php
			button.gif
			editnews.php
			flagchk.php
			install.php
			MgLine.zip
			news.php
			noteBBSa.php
			oekakiBBSa.php
			README.txt
			redirect.php
			tdesign.php
			tedit.php
			tedita.php
			template.php
			test.php
			tnew.php
			tnewa.php
			versionchk.php
	*/


	// Upgrade config
	include ('config.php');
	include ('dbconn.php');
	$BBStitle  = addslashes(stripslashes($BBStitle));
	$BBSauthor = addslashes(stripslashes($BBSauthor));
	$CFG_previewTitle = addslashes(stripslashes($CFG_previewTitle));
	$okurl        = strip_path($okurl);
	$OPpics       = strip_path($OPpics);
	$templatepath = strip_path($templatepath);
	$resourcepath = strip_path($resourcepath);
	if ($BBSadult != 'yes') {
		$BBSadult     = 'no';
	}
	if ($useChat != 'no') {
		$useChat = 'yes';
	}

	if ($perpage == '') {
		// Only set defaults if not already updated.
		$BBSauthor = '';
		$OPPrefix = '';
		$perpage = 25;
		$xMin = 50;
		$yMin = 50;
		$aCanvas = 250000;
		$requireart = 'no';
		$CFG_previewImg   = 'preview.png';
		$CFG_previewTitle = 'Canvas Preview';
		$templatepath = 'templates';
		$resourcepath = 'resource';
		$useChat = 'yes';
	}

	if ($_POST['updatetemplate'] != '') {
		$template  = $_POST['updatetemplate'];
		$BBSauthor = w_gpc('BBSauthor', 'html');
		$BBSadult  = $_POST['BBSadult'];
	}

	// Fix Wacintaki alpha version
	if (file_exists('resource') && empty ($resourcepath)) {
		$resourcepath = 'resource';
	}

	$main_configuration = <<<EOF
<?php // Include only

	\$BBStitle  = '$BBStitle';
	\$BBSauthor = '$BBSauthor';
	\$BBSadult  = '$BBSadult';
	\$eaddr     = '$eaddr';
	\$okurl     = '$okurl';

	\$saltenc    = '$saltenc';
	\$OPpics     = '$OPpics';
	\$archivedir = '$archivedir';
	\$OPPrefix   = '$OPPrefix';

	\$killuser   = '$killuser';
	\$approval   = '$approval';
	\$requireart = '$requireart';
	\$guestpost  = '$guestpost';
	\$drawaccess = '$drawaccess';
	\$animationaccess = '$animationaccess';

	\$language     = '$language';
	\$template     = '$template';
	\$templatepath = '$templatepath';
	\$resourcepath = '$resourcepath';

	\$pstore  = '$pstore';
	\$ppage   = '$ppage';
	\$perpage = '$perpage';

	\$CFG_previewImg   = '$CFG_previewImg';
	\$CFG_previewTitle = '$CFG_previewTitle';
	\$xDefault = '$xDefault';
	\$yDefault = '$yDefault';
	\$xCanvas  = '$xCanvas';
	\$yCanvas  = '$yCanvas';
	\$xMin     = '$xMin';
	\$yMin     = '$yMin';
	\$aCanvas  = '$aCanvas';
	\$anisize  = '$anisize';

	\$useChat  = '$useChat';
	\$chatMax  = '$chatMax';
	\$chatDisp = '$chatDisp';

	\$jpgcompression  = '$jpgcompression';
	\$jpgcompressqual = '$jpgcompressqual';

?>
EOF;
//<?


	// Write config file
	$fp = @fopen ('config.php', 'w');
	if ($fp) {
		fwrite ($fp, $main_configuration);
		fclose ($fp);
	} else {
		web_line('<strong>STOP</strong>:  Could not open config file for writing.  Check your server permissions');

		// Nothing done yet, so we can bail safely
		return 0;
	}


	// Update templates/languages
	{
		$newtemplate = w_gpc('updatetemplate');
		if ($newtemplate == '') {
			$newtemplate = 'Default';
		}

		// Template
		$result = mysql_query ("ALTER TABLE ".$OekakiPoteto_MemberPrefix."oekaki CHANGE templatesel templatesel VARCHAR(255) DEFAULT '".$newtemplate."'");
		sql_check($result);

		$result = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET templatesel='".$newtemplate."'");
		sql_check($result);


		// Language
		if (file_exists ('language/'.$language.'.php')) {
			$result = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET language='".$language."'");
		} else {
			$result = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET language='english'");
		}
		sql_check($result);


		mysql_close ($dbconn);
	}


	// Verify CHMOD
	if (!check_write($OPpics, 755)) {
		web_line('ERROR: Picture directory "'.$OPpics.'" is not writable!  CHMOD the folder to 755 or 777 after the updater has finished.');
	}

	if (!check_write('templates', 755)) {
		web_line('ERROR: Templates directory is not writable!  CHMOD the folder to 755 or 777 after the updater has finished.');
	}


	// Remove files if they exist
	{
		@unlink('announce.php');
		@unlink('arecover.php');
		@unlink('button.gif');
		@unlink('editnews.php');
		@unlink('flagchk.php');
		if ($CFG_update['debug'] == 0) @unlink('install.php');
		@unlink('MgLine.zip');
		@unlink('news.php');
		@unlink('noteBBSa.php');
		@unlink('oekakiBBSa.php');
		@unlink('README.txt');
		@unlink('redirect.php');
		@unlink('tdesign.php');
		@unlink('tedit.php');
		@unlink('tedita.php');
		@unlink('template.php');
		@unlink('test.php');
		@unlink('tnew.php');
		@unlink('tnewa.php');
		@unlink('versionchk.php');
	}


	// Complete!
	if ($CFG_update['status'] == 0) {
		web_line('Update finished: Wacintaki Poteto v1.0.0');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END op500_to_wac100


function wac100_to_wac110($force) {
	global $CFG_update;

	// Wacintaki Poteto 1.0.x to Wacintaki Poteto 1.1.0
	web_line('Update from Wacintaki Poteto v1.0.0 to v1.1.0...');

	/* We need to do the following:
		1) Add new values to config file:
			$CFG_version     = 'wac-1.1.0';
			$killreg         = '-1'; //WIP
			$guestdraw       = 'no'; //WIP
			$guestadult      = 'yes';
			$picLimit        = '2';  //WIP
			$picLimitTime    = '24'; //WIP
			$CFG_pornImg     = 'pr0n.png';
			$useThumb        = '2';
			$forceThumb      = 'no';
			$thumbBytes      = '100000';
			$thumbT          = '120';
			$thumbR          = '250';
			$thumbJPG        = '75'; //WIP
			$jpgcompression  = 'no'; //WIP
			$jpgcompressqual = '85'; //WIP
			$useMailbox      = 'no';
			$useSuggest      = 'no'; //WIP
			$suggestMax      = '50'; //WIP
			$suggestApprove  = 'no'; //WIP

		2) Add new values to database (finally):
			oekaki: thumbview
			oekaki: screensize
			oekakidta: threadlock
			oekakidta: px
			oekakidta: py
			oekakidta: ptype
			oekakidta: ttype
			oekakidta: rtype
			oekakidta: usethumb
			oekakimisc: miscstring

		3) Remove files if they exist:
			compress.php
			install.php
	*/

	// Update DB
	include ('config.php');
	$BBStitle  = addslashes(stripslashes($BBStitle));
	$BBSauthor = addslashes(stripslashes($BBSauthor));
	$CFG_previewTitle = addslashes(stripslashes($CFG_previewTitle));

	include ('dbconn.php');
	{
		// Ability to lock threads.
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD threadlock TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Picture sizes
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD px MEDIUMINT DEFAULT 0');
		sql_check($result);
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD py MEDIUMINT DEFAULT 0');
		sql_check($result);

		// Yes, VARCHAR for datatype (file extension) is better than INT as it allows for file uploads, not just pictures.  It also makes things easier for future mods.
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD ptype VARCHAR(4)');
		sql_check($result);
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD ttype VARCHAR(4)');
		sql_check($result);
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD rtype VARCHAR(4)');
		sql_check($result);

		// Records if thumbnail is available (built), unavailable, or should not be used (dimmentions or filesize too small, etc).
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_Prefix.'oekakidta ADD usethumb TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Thumbnail mode for each user
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_MemberPrefix.'oekaki ADD thumbview TINYINT NOT NULL DEFAULT 0');
		sql_check($result);

		// Screen size for each user
		$result = mysql_query ('ALTER TABLE '.$OekakiPoteto_MemberPrefix.'oekaki ADD screensize MEDIUMINT NOT NULL DEFAULT 800');
		sql_check($result);

		// Version control
		$result = mysql_query ('ALTER TABLE {$OekakiPoteto_Prefix}oekakimisc ADD miscstring VARCHAR(14)');
		sql_check($result);

		// Done with DB update
		$result = mysql_query ("INSERT INTO {$OekakiPoteto_Prefix}oekakimisc VALUES (2, 'dbversion', 0, 'wac-1.1.0')");
		sql_check($result);
	}

	// Set DB values
	{
		// Sets view mode for each user to automatic (0)
		$result = mysql_query ('UPDATE '.$OekakiPoteto_MemberPrefix.'oekaki SET picview=0');
		sql_check($result);
	}

	// Update config file
	{
		if (extension_loaded ('gd')) {
			$thumbnail_detect = 2;
		} else {
			$thumbnail_detect = 0;
		}

		$main_configuration = <<<EOF
<?php // Include only

	\$CFG_version = 'wac-1.1.0';

	\$BBStitle  = '$BBStitle';
	\$BBSauthor = '$BBSauthor';
	\$BBSadult  = '$BBSadult';
	\$eaddr     = '$eaddr';
	\$okurl     = '$okurl';

	\$saltenc    = '$saltenc';
	\$OPpics     = '$OPpics';
	\$archivedir = '$archivedir';
	\$OPPrefix   = '$OPPrefix';

	\$killuser   = '$killuser';
	\$killreg    = '-1'; //WIP
	\$approval   = '$approval';
	\$requireart = '$requireart';
	\$guestpost  = '$guestpost';
	\$guestdraw  = 'no'; //WIP
	\$guestadult = 'yes';
	\$drawaccess = '$drawaccess';
	\$animationaccess = '$animationaccess';

	\$language     = '$language';
	\$template     = '$template';
	\$templatepath = '$templatepath';
	\$resourcepath = '$resourcepath';

	\$pstore   = '$pstore';
	\$ppage    = '$ppage';
	\$perpage  = '$perpage';
	\$picLimit     = '2';  //WIP
	\$picLimitTime = '24'; //WIP

	\$CFG_pornImg      = 'pr0n.png';
	\$CFG_previewImg   = '$CFG_previewImg';
	\$CFG_previewTitle = '$CFG_previewTitle';
	\$xDefault = '$xDefault';
	\$yDefault = '$yDefault';
	\$xCanvas  = '$xCanvas';
	\$yCanvas  = '$yCanvas';
	\$xMin     = '$xMin';
	\$yMin     = '$yMin';
	\$aCanvas  = '$aCanvas';
	\$anisize  = '$anisize';

	\$useThumb   = '$thumbnail_detect';
	\$forceThumb = 'no';
	\$thumbBytes = '100000';
	\$thumbT     = '120';
	\$thumbR     = '250';
	\$thumbJPG   = '75'; //WIP
	\$jpgcompression   = 'no'; //WIP
	\$jpgcompressqual  = '85'; //WIP

	\$useMailbox = 'yes';

	\$useChat  = '$useChat';
	\$chatMax  = '$chatMax';
	\$chatDisp = '$chatDisp';

	\$useSuggest     = 'no'; //WIP
	\$suggestMax     = '50'; //WIP
	\$suggestApprove = 'no'; //WIP
?>
EOF;
//<?

		// Write the config file
		if ($fp = fopen ('config.php', 'w')) {
			fwrite ($fp, $main_configuration);
			fclose ($fp);
		} else {
			web_line('<strong>ERROR</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			$CFG_update['status'] = 1;
		}
	}

	// Build ptypes
	{
		/*
		It's possible the source image can be a JPEG or PNG but there's no way to tell which is the primarty image unless we search the filesystem for both.  This is partly my fault thanks to my upload script which allows people to upload JPEGs if $jpgcompression is enabled (normally, Poteto can only use PNGs).

		If a PNG exists, we always use that, but if a JPEG exists exclusively, we use that instead.  Rather than bang the filesys, we'll read the directory and do lots of string searching to find out what's available.  There's probably a better way using stacks, but I'm stupid.  :)
		*/

		// Read pic folder.  Collect names of PNGs and JPEGs
		$handle = opendir($OPpics);
		$prefix_len = strlen ($OPPrefix) + 1;
		while ($file = readdir($handle)){
			if (substr ($file, -4) == '.png' || substr ($file, -4) == '.jpg') {
				// Image file.  If not a thumbnail, queue it.
				$thumbtest = substr ($file, 0, $prefix_len);
				if ($thumbtest != $OPPrefix.'t' && $thumbtest != $OPPrefix.'r') {
					$files[] = $file;
				}
			}
		}
		closedir($handle);

		// Collect database entries and let's sort things out
		$result = mysql_query ('SELECT ID_2, PIC_ID, px, py, ptype FROM '.$OekakiPoteto_Prefix.'oekakidta');
		$total_rows = mysql_num_rows ($result);

		for ($i = 0; $i < $total_rows; $i++) {
			$row = mysql_fetch_array ($result);

			// If ptype is occupied, we do nothing unless a force is needed.
			// Rather than juggle filetypes, assume valid extension is > 2 char.
			if (strlen ($row['ptype']) < 2 || $force == true) {

				// Get the pic's datatype
				// Use PNG if avaiable, otherwise, use JPEG ("thumbnail" in Poteto)
				$my_pre = $OPPrefix.$row['PIC_ID'].'.';
				$found_img = false;

				if (in_array ($my_pre.'png', $files)) {
					// Found PNG!
					$row['ptype'] = 'png';
					$found_img = true;
				} elseif (in_array ($my_pre.'jpg', $files)) {
					// No PNG.  Found JPG!
					$row['ptype'] = 'jpg';
					$found_img = true;
				}

				// Verify.  Check ptype first to avoid filesys slowdown
				$image = $OPpics.'/'.$OPPrefix.$row['PIC_ID'].'.'.$row['ptype'];
				if ($found_img) {
					// Assume image may be corrupt
					$sizes = @GetImageSize ($image);
					if (!$sizes) {
						$sizes = array ($xDefault, $yDefault);
					}

					// OK, we've got all our info.  Insert it into DB
					if ($debug == 1) {
						web_line("PIC: {$row['PIC_ID']}, <strong>X: {$sizes[0]}, Y: {$sizes[1]}</strong>, TYPE: {$row['ptype']}");
					} else {
						$result2 = mysql_query ("UPDATE {$OekakiPoteto_Prefix}oekakidta SET px={$sizes[0]}, py={$sizes[1]}, ptype='{$row['ptype']}' WHERE PIC_ID='{$row['PIC_ID']}'");
						sql_check($result2);
					}
				} else {
					// D'OH!  Picture missing!
					web_line('<strong>NOTE:</strong> missing image "'.$OPPrefix.$row['PIC_ID'].'.xxx" for post '.$row['ID_2'].'!  Consider deleting the post rather than trying to FTP a picture into the pictures folder.');
				}
			}
		}
	}
	// End build ptypes


	// Remove files if they exist
	{
		@unlink('compress.php');
		if ($CFG_update['debug'] == 0) @unlink('install.php');
	}


	// Remove obsolete templates
	if ($handle = opendir('templates')) {
		/* This is the correct way to loop over the directory. */
		while (false !== ($file = readdir($handle))) {
			if(substr($file,-4) == '.css'){
				if ($file != 'Default.css' && $file != 'Install.css') {
					@unlink ('templates/'.$file);
				}
			}
		}
		closedir($handle);
	}


	// Complete!
	if ($CFG_update['status'] == 0 || $force == true) {
		web_line('Update finished: Wacintaki Poteto v1.1.0');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END wac100_to_wac110


function wac110_to_wac111() {
	global $CFG_update;

	// Wacintaki Poteto 1.1.0 to Wacintaki Poteto 1.1.1
	web_line('Update from Wacintaki Poteto v1.1.0 to v1.1.1...');

	/* We need to do the following:
		1) Add new values to config file:
			$uploadaccess = 'no';
			$max_upload = '500000';

		2) Give all admins upload flag.  Not strictly needed, but proper.

		3) Remove files if they exist:
			install.php
	*/


	include ('dbconn.php');

	// Update upload flags.  Errors are no big deal.
	$result = mysql_query ("SELECT usrname, usrflags FROM ".$OekakiPoteto_MemberPrefix."oekaki");
	if ($result) {
		$rows = mysql_num_rows ($result);
		$i = 0;

		while ($i < $rows) {
			$user_admin = 0;
			$row = mysql_fetch_array ($result);
			$flags = eregi_replace('U', '', $row['usrflags']);
			if (strstr($row['usrflags'], 'O') || strstr($row['usrflags'], 'S') || strstr($row['usrflags'], 'A')) {
				$user_admin = 1;
			}

			if ($_POST['upload_grant'] == 'yes' && strstr($row['usrflags'], 'D')) {
				$flags .= 'U';
				$result2 = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET usrflags='$flags' WHERE usrname='{$row['usrname']}'");

				if (!$result2) {
					echo "Can't give user \"{$row['usrname']}\" upload flag (may ignore for all users, including admins).<br />\n";
				}
			} elseif ($user_admin) {
				$flags .= 'U';
				$result2 = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET usrflags='$flags' WHERE usrname='{$row['usrname']}'");

				if (!$result2) {
					echo "Can't give admin \"{$row['usrname']}\" upload flag (ignore).<br />\n";
				}
			} elseif ($_POST['upload_grant'] == 'strip' && $user_admin == 0) {
				$result2 = mysql_query ("UPDATE ".$OekakiPoteto_MemberPrefix."oekaki SET usrflags='$flags' WHERE usrname='{$row['usrname']}'");

				if (!$result2) {
					echo "<strong>ERROR:</strong>  can't remove upload flag for \"{$row['usrname']}\".<br />\n";
				}
			}
			$i++;
		}
	}
	mysql_close ($dbconn);



	// Update config file
	if (empty ($_POST['upload_flag'])) {
		$_POST['upload_flag'] = 'no';
	}

	include ('config.php');
	$BBStitle  = addslashes(stripslashes($BBStitle));
	$BBSauthor = addslashes(stripslashes($BBSauthor));
	$CFG_previewTitle = addslashes(stripslashes($CFG_previewTitle));

	{
		if (extension_loaded ('gd')) {
			$thumbnail_detect = 2;
		} else {
			$thumbnail_detect = 0;
		}

		$main_configuration = <<<EOF
<?php // Include only

	\$CFG_version = 'wac-1.1.1';

	\$BBStitle  = '$BBStitle';
	\$BBSauthor = '$BBSauthor';
	\$BBSadult  = '$BBSadult';
	\$eaddr     = '$eaddr';
	\$okurl     = '$okurl';

	\$saltenc    = '$saltenc';
	\$OPpics     = '$OPpics';
	\$archivedir = '$archivedir';
	\$OPPrefix   = '$OPPrefix';

	\$killuser   = '$killuser';
	\$killreg    = '-1'; //WIP
	\$approval   = '$approval';
	\$requireart = '$requireart';
	\$guestpost  = '$guestpost';
	\$guestdraw  = 'no'; //WIP
	\$guestadult = 'yes';
	\$drawaccess      = '$drawaccess';
	\$animationaccess = '$animationaccess';
	\$uploadaccess    = '{$_POST['upload_flag']}';

	\$language     = '$language';
	\$template     = '$template';
	\$templatepath = '$templatepath';
	\$resourcepath = '$resourcepath';

	\$pstore   = '$pstore';
	\$ppage    = '$ppage';
	\$perpage  = '$perpage';
	\$picLimit     = '2';  //WIP
	\$picLimitTime = '24'; //WIP

	\$CFG_pornImg      = 'pr0n.png';
	\$CFG_previewImg   = '$CFG_previewImg';
	\$CFG_previewTitle = '$CFG_previewTitle';
	\$xDefault   = '$xDefault';
	\$yDefault   = '$yDefault';
	\$xCanvas    = '$xCanvas';
	\$yCanvas    = '$yCanvas';
	\$xMin       = '$xMin';
	\$yMin       = '$yMin';
	\$aCanvas    = '$aCanvas';
	\$anisize    = '$anisize';
	\$max_upload = '500000';

	\$useThumb   = '$thumbnail_detect';
	\$forceThumb = 'no';
	\$thumbBytes = '100000';
	\$thumbT     = '120';
	\$thumbR     = '250';
	\$thumbJPG   = '75'; //WIP
	\$jpgcompression   = 'no'; //WIP
	\$jpgcompressqual  = '85'; //WIP

	\$useMailbox = 'yes';

	\$useChat  = '$useChat';
	\$chatMax  = '$chatMax';
	\$chatDisp = '$chatDisp';

	\$useSuggest     = 'no'; //WIP
	\$suggestMax     = '50'; //WIP
	\$suggestApprove = 'no'; //WIP
?>
EOF;
//<?

		// Write the config file
		if ($fp = fopen ('config.php', 'w')) {
			fwrite ($fp, $main_configuration);
			fclose ($fp);
		} else {
			web_line('<strong>ERROR</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			$CFG_update['status'] = 1;
		}
	}


	// Remove files if they exist
	if ($CFG_update['debug'] == 0) @unlink('install.php');


	// Complete!
	if ($CFG_update['status'] == 0) {
		web_line('Update finished: Wacintaki Poteto v1.1.1');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END wac110_to_wac111


function wac111_to_wac120() {
	global $CFG_update;

	// Wacintaki Poteto 1.1.1 to Wacintaki Poteto 1.2.0
	web_line('Update from Wacintaki Poteto 1.1.1 to 1.2.0...');

	/* We need to do the following, in order of safety:
		1) Dump op_oekakikill

		2) Add password field to op_oekakidta (not used yet)

		3) Reset all default templates in op_oekaki

		4) Add and set member ranks to op_oekaki

		5) Change config file variables to keyed array

		6) Update admin flags to use ranks
	*/


	include ('dbconn.php');
	include ('config.php');

	// Update database
	{
		// Dump oekakikill (ignore errors)
		$result = mysql_query ("DROP TABLE {$OekakiPoteto_MemberPrefix}oekakikill");

		// Add password field
		$result = mysql_query ("ALTER TABLE {$OekakiPoteto_Prefix}oekakidta ADD password text");

		// Flush default templates if using old config file
		if (isset ($template) && !empty ($template)) {
			$result = mysql_query ("UPDATE {$OekakiPoteto_MemberPrefix}oekaki SET templatesel='' WHERE templatesel='$template'");
		}

		// Add ranks if they don't exist
		$result = mysql_query ("SELECT * FROM {$OekakiPoteto_MemberPrefix}oekaki");
		$junk = mysql_fetch_array ($result);
		if (!isset ($junk['rank'])) {
			$result = mysql_query ("ALTER TABLE {$OekakiPoteto_MemberPrefix}oekaki ADD rank TINYINT NOT NULL DEFAULT 0");

			if (!$result) {
				web_line('<strong>STOP</strong>: Unable to add admin ranks to database!  Check the database for permission issues or try again.');
				mysql_close ($dbconn);
				return 0;
			}
		}

		// Set ranks
		$result_names = mysql_query ("SELECT usrname, usrflags FROM {$OekakiPoteto_MemberPrefix}oekaki");
		$all_rows = mysql_num_rows ($result_names);
		for ($i = 0; $i < $all_rows; $i++) {
			$rank = 0;

			$row = mysql_fetch_array ($result_names);
			if (strstr ($row['usrflags'], 'A'))
				$rank = RANK_ADMIN;
			if (strstr ($row['usrflags'], 'S'))
				$rank = RANK_SADMIN;
			if (strstr ($row['usrflags'], 'O'))
				$rank = RANK_OWNER;
			if ($rank > 0) {
				$result = mysql_query ("UPDATE {$OekakiPoteto_MemberPrefix}oekaki SET rank=$rank WHERE usrname='{$row['usrname']}'");

				if (!$result) {
					web_line('<strong>STOP</strong>:  Could not set admin rank for '.$row['usrname'].'!  Check your database for a flakey connection.');
					return 0;
				}
			}
		}

		// Done!  Update the DB version
		$result = mysql_query ("UPDATE {$OekakiPoteto_Prefix}oekakimisc SET miscstring='1.2.0' WHERE miscname='dbversion'");
	}


	// Update config file if not updated already
	if (!isset ($cfg['version']) && isset ($CFG_version)) {
		$BBStitle  = addslashes (stripslashes ($BBStitle));
		$BBSauthor = addslashes (stripslashes ($BBSauthor));
		$CFG_previewTitle = addslashes (stripslashes ($CFG_previewTitle));

		$main_configuration = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '1.2.0';

	\$cfg['op_title']  = '{$BBStitle}';
	\$cfg['op_author'] = '{$BBSauthor}';
	\$cfg['op_adult']  = '{$BBSadult}';
	\$cfg['op_email']  = '{$eaddr}';
	\$cfg['op_url']    = '{$okurl}';

	\$cfg['salt']    = '{$saltenc}';
	\$cfg['op_pics'] = '{$OPpics}';
	\$cfg['op_pre']  = '{$OPPrefix}';

	\$cfg['kill_user']   = '{$killuser}';
	\$cfg['kill_reg']    = '15';
	\$cfg['approval']    = '{$approval}';
	\$cfg['require_art'] = '{$requireart}';
	\$cfg['guest_post']  = '{$guestpost}';
	\$cfg['guest_draw']  = '{$guestdraw}'; //WIP
	\$cfg['guest_adult'] = '{$guestadult}';
	\$cfg['reg_draw']    = '{$drawaccess}';
	\$cfg['reg_anim']    = '{$animationaccess}';
	\$cfg['reg_upload']  = '{$uploadaccess}';
	\$cfg['reg_rules']   = 'no';

	\$cfg['language'] = '{$language}';
	\$cfg['template'] = '{$template}';
	\$cfg['tpl_path'] = '{$templatepath}';
	\$cfg['res_path'] = '{$resourcepath}';

	\$cfg['pic_store']  = '{$pstore}';
	\$cfg['pic_pages']  = '{$ppage}';
	\$cfg['menu_pages'] = '{$perpage}';
	\$cfg['pic_limit']      = '{$picLimit}'; //WIP
	\$cfg['pic_limit_time'] = '{$picLimitTime}'; //WIP

	\$cfg['porn_img']      = '{$CFG_pornImg}';
	\$cfg['preview_img']   = '{$CFG_previewImg}';
	\$cfg['preview_title'] = '{$CFG_previewTitle}';
	\$cfg['def_x']    = '{$xDefault}';
	\$cfg['def_y']    = '{$yDefault}';
	\$cfg['canvas_x'] = '{$xCanvas}';
	\$cfg['canvas_y'] = '{$yCanvas}';
	\$cfg['min_x']    = '{$xMin}';
	\$cfg['min_y']    = '{$yMin}';
	\$cfg['canvas_a'] = '{$aCanvas}';
	\$cfg['max_anim'] = '{$anisize}';
	\$cfg['max_pic']  = '{$max_upload}';

	\$cfg['use_thumb']   = '{$useThumb}';
	\$cfg['force_thumb'] = '{$forceThumb}';
	\$cfg['thumb_bytes'] = '{$thumbBytes}';
	\$cfg['thumb_t']     = '{$thumbT}';
	\$cfg['thumb_r']     = '{$thumbR}';
	\$cfg['thumb_jpg']   = '{$thumbJPG}';

	\$cfg['use_mailbox'] = '{$useMailbox}';

	\$cfg['use_chat']   = '{$useChat}';
	\$cfg['chat_max']   = '{$chatMax}';
	\$cfg['chat_pages'] = '{$chatDisp}';

	\$cfg['smilies'] = 'yes';
?>
EOF;
//<?

		// Write the config file
		if ($fp = fopen ('config.php', 'w')) {
			fwrite ($fp, $main_configuration);
			fclose ($fp);
		} else {
			web_line('<strong>STOP</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			mysql_close ($dbconn);
			return 0;
		}
	}

	// Update the admin flags so only ranks are used
	$result_names = mysql_query ("SELECT usrname, usrflags FROM {$OekakiPoteto_MemberPrefix}oekaki");
	$all_rows = mysql_num_rows ($result_names);
	for ($i = 0; $i < $all_rows; $i++) {
		$row = mysql_fetch_array ($result_names);

		if (strstr ($row['usrflags'], 'A')) {
			$new_flags = str_replace (array ('O', 'S', 'A'), '', $row['usrflags']);

			$result = mysql_query ("UPDATE {$OekakiPoteto_MemberPrefix}oekaki SET usrflags='$new_flags' WHERE usrname='{$row['usrname']}'");
		}
	}


	mysql_close ($dbconn);


	// Check for rules, banner
	if (!file_exists ('resource/rules.php')) {
		$fp = @fopen ('resource/rules.php', 'w');
		if ($fp) {
			fwrite ($fp, "<p>\nNetiquette applies.  Ask the admin if you have any questions.\n</p>");
			fclose ($fp);
		}
	}
	if (!file_exists ('resource/banner.php')) {
		$fp = @fopen ('resource/rules.php', 'w');
		fclose ($fp);
	}


	@unlink ('online.php');
	@unlink ('version.php');
	if ($CFG_update['debug'] == 0) @unlink ('install.php');


	// Complete!
	if ($CFG_update['status'] == 0) {
		web_line('Update finished: Wacintaki Poteto 1.2.0');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END wac111_to_wac120


function wac120_to_wac130() {
	global $CFG_update;

	// Wacintaki Poteto 1.2.0 to Wacintaki Poteto 1.3.0
	web_line('Update from Wacintaki Poteto 1.2.0 to 1.3.0...');

	/* We need to do the following, in order of safety or processing time:
		1) Reset member picture counts if less than 0 (bug in 1.2.0 upload script)

		2) Create avatars folder

		3) Update database to be MySQL 5 compliant (correct default values)

		4) Add avatar field to op_oekaki, origart to op_oekakidta

		5) Update db version stamp to 1.3.0

		6) Update main piccount to lowest non-conflict value (new sorting system)

		7) Update config file to 1.3.0, and change version stamp

		8) Clean up orphaned picture/temp files leftover from Wac 1.2.x
	*/


	include ('dbconn.php');
	include ('config.php');


	// Fix member picture count
	$result = mysql_query ("UPDATE {$OekakiPoteto_MemberPrefix}oekaki SET piccount=0 WHERE piccount < 0");

	// Add avatars folder
	if (!file_exists ('avatars')) {
		if (! @mkdir ('avatars', 0755)) {
			web_line('<strong>STOP</strong>: Could not create "avatars" folder.  Check your server permissions.');
			mysql_close ($dbconn);
			return 0;
		}
	}
	if (!check_write('avatars', 755)) {
		web_line('ERROR: Avatars directory is not writable!  CHMOD the folder to 755 or 777 after the updater has finished.');
	}


	// Update database
	$result = mysql_query ("SELECT miscstring FROM {$OekakiPoteto_Prefix}oekakimisc WHERE miscname='dbversion'");
	if ($result && mysql_result ($result, 0) == '1.2.0') {

		// Update default values for MySQL 5
		// MySQL 5 now enforces the rule that any field with a "NOT NULL" clause *must* have a default value
		$pre = $OekakiPoteto_Prefix;
		$mem = $OekakiPoteto_MemberPrefix;

		$q   = array();
		$q[] = "{$mem}oekaki CHANGE templatesel templatesel varchar(255) NOT NULL DEFAULT ''";
		$q[] = "{$mem}oekakichat CHANGE comment comment TINYtext";
		$q[] = "{$mem}oekakichat CHANGE hostname hostname text";
		$q[] = "{$mem}oekakichat CHANGE `IP` `IP` text";
		$q[] = "{$mem}oekakionline CHANGE onlineIP onlineIP text";
		$q[] = "{$mem}oekakionline CHANGE onlineboard onlineboard text";
		$q[] = "{$mem}oekakimailbox CHANGE subject subject text";
		$q[] = "{$mem}oekakimailbox CHANGE body body LONGtext";
		$q[] = "{$mem}oekakichatmod CHANGE comment comment TINYtext";
		$q[] = "{$mem}oekakichatmod CHANGE hostname hostname text";
		$q[] = "{$mem}oekakichatmod CHANGE `IP` `IP` text";
		$q[] = "{$mem}oekakichatmodusers CHANGE usrname usrname VARCHAR(255) NOT NULL DEFAULT ''";

		$q[] = "{$pre}oekakicmt CHANGE comment comment text";
		$q[] = "{$pre}oekakicmt CHANGE hostname hostname text";
		$q[] = "{$pre}oekakicmt CHANGE `IP` `IP` text";
		$q[] = "{$pre}oekakidta CHANGE comment comment text";
		$q[] = "{$pre}oekakidta CHANGE hostname hostname text";
		$q[] = "{$pre}oekakidta CHANGE `IP` `IP` text";
		$q[] = "{$pre}oekakidta CHANGE password password text";

		foreach ($q as $query) {
			$result = mysql_query ('ALTER TABLE '.$query);
			if (!$result) {
				web_line('MySQL5_Fix: '.mysql_error());
			}
		}


		// Add avatar field
		$result = mysql_query ("ALTER TABLE {$OekakiPoteto_MemberPrefix}oekaki ADD avatar VARCHAR(128) NOT NULL DEFAULT ''");

		// Add origart field
		$result = mysql_query ("ALTER TABLE {$OekakiPoteto_Prefix}oekakidta ADD origart VARCHAR(255) NOT NULL DEFAULT ''");

		// Done!  Update the DB version
		$result = mysql_query ("UPDATE {$OekakiPoteto_Prefix}oekakimisc SET miscstring='1.3.0' WHERE miscname='dbversion'");
	}


	// Update piccount for new sorting system (about time)
	$new_piccount = 0;
	$result = mysql_query ("SELECT MAX(PIC_ID) FROM {$OekakiPoteto_Prefix}oekakidta");
	if ($result)
		$new_piccount = intval (mysql_result ($result, 0));

	if ($new_piccount == 0)
		$new_piccount = $cfg['pic_store'];

	$result = mysql_query ("UPDATE {$OekakiPoteto_Prefix}oekakimisc SET miscvalue={$new_piccount} WHERE miscname='piccount'");
	if (!$result) {
		web_line('<strong>STOP</strong>: Could not update piccount (current picture number) for new sorting system.  This is important, so check your SQL connection and/or try again.');
		mysql_close ($dbconn);
		return 0;
	}


	// Update config file if not updated already
	if ($cfg['version'] == '1.2.0') {
		$BBStitle  = addslashes ($cfg['op_title']);
		$BBSauthor = addslashes ($cfg['op_author']);
		$CFG_previewTitle = addslashes ($cfg['preview_title']);

		$main_configuration = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '1.3.0';

	\$cfg['op_title']  = '{$BBStitle}';
	\$cfg['op_author'] = '{$BBSauthor}';
	\$cfg['op_adult']  = '{$cfg['op_adult']}';
	\$cfg['op_email']  = '{$cfg['op_email']}';
	\$cfg['op_url']    = '{$cfg['op_url']}';

	\$cfg['salt']    = '{$cfg['salt']}';
	\$cfg['op_pics'] = '{$cfg['op_pics']}';
	\$cfg['op_pre']  = '{$cfg['op_pre']}';

	\$cfg['kill_user']   = '{$cfg['kill_user']}';
	\$cfg['kill_reg']    = '{$cfg['kill_reg']}';
	\$cfg['approval']    = '{$cfg['approval']}';
	\$cfg['draw_immune'] = 'no';
	\$cfg['require_art'] = '{$cfg['require_art']}';
	\$cfg['guest_post']  = '{$cfg['guest_post']}';
	\$cfg['guest_draw']  = '{$cfg['guest_draw']}'; //WIP
	\$cfg['guest_adult'] = '{$cfg['guest_adult']}';
	\$cfg['reg_draw']    = '{$cfg['reg_draw']}';
	\$cfg['reg_anim']    = '{$cfg['reg_anim']}';
	\$cfg['reg_upload']  = '{$cfg['reg_upload']}';
	\$cfg['reg_rules']   = '{$cfg['reg_rules']}';

	\$cfg['language'] = '{$cfg['language']}';
	\$cfg['template'] = '{$cfg['template']}';
	\$cfg['tpl_path'] = '{$cfg['tpl_path']}';
	\$cfg['res_path'] = '{$cfg['res_path']}';

	\$cfg['pic_store']  = '{$cfg['pic_store']}';
	\$cfg['pic_pages']  = '{$cfg['pic_pages']}';
	\$cfg['menu_pages'] = '{$cfg['menu_pages']}';
	\$cfg['pic_limit']      = '{$cfg['pic_limit']}'; //WIP
	\$cfg['pic_limit_time'] = '{$cfg['pic_limit_time']}'; //WIP

	\$cfg['porn_img']      = '{$cfg['porn_img']}';
	\$cfg['preview_img']   = '{$cfg['preview_img']}';
	\$cfg['preview_title'] = '{$CFG_previewTitle}';
	\$cfg['def_x']    = '{$cfg['def_x']}';
	\$cfg['def_y']    = '{$cfg['def_y']}';
	\$cfg['canvas_x'] = '{$cfg['canvas_x']}';
	\$cfg['canvas_y'] = '{$cfg['canvas_y']}';
	\$cfg['min_x']    = '{$cfg['min_x']}';
	\$cfg['min_y']    = '{$cfg['min_y']}';
	\$cfg['canvas_a'] = '{$cfg['canvas_a']}';
	\$cfg['max_anim'] = '{$cfg['max_anim']}';
	\$cfg['max_pic']  = '{$cfg['max_pic']}';

	\$cfg['use_thumb']   = '{$cfg['use_thumb']}';
	\$cfg['force_thumb'] = '{$cfg['force_thumb']}';
	\$cfg['thumb_bytes'] = '{$cfg['thumb_bytes']}';
	\$cfg['thumb_t']     = '{$cfg['thumb_t']}';
	\$cfg['thumb_r']     = '{$cfg['thumb_r']}';
	\$cfg['thumb_jpg']   = '{$cfg['thumb_jpg']}';

	\$cfg['use_mailbox'] = '{$cfg['use_mailbox']}';

	\$cfg['use_chat']   = '{$cfg['use_chat']}';
	\$cfg['chat_max']   = '{$cfg['chat_max']}';
	\$cfg['chat_pages'] = '{$cfg['chat_pages']}';

	\$cfg['smilies'] = 'yes';

	\$cfg['use_avatars']   = 'yes';
	\$cfg['use_c_avatars'] = 'no';
	\$cfg['avatar_folder'] = 'avatars';
	\$cfg['avatar_x'] = '50';
	\$cfg['avatar_y'] = '50';

	\$cfg['public_retouch']   = 'yes';
	\$cfg['safety_storetime'] = '10';
	\$cfg['safety_saves']     = 'yes';
	\$cfg['self_bump']        = 'yes';
?>
EOF;
//<?

		// Write the config file
		if ($fp = fopen ('config.php', 'w')) {
			fwrite ($fp, $main_configuration);
			fclose ($fp);
		} else {
			web_line('<strong>STOP</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
			mysql_close ($dbconn);
			return 0;
		}
	}

	// Remove files if they exist
	if ($CFG_update['debug'] == 0) @unlink ('install.php');


	// Clean orphaned images, if any.  Discard return code
	clean_images($CFG_update['debug']);


	// Complete!
	mysql_close ($dbconn);

	if ($CFG_update['status'] == 0) {
		web_line('Update finished: Wacintaki Poteto 1.3.0');
		web_line('');

		return 1;
	} else {
		return 0;
	}
}
// END wac120_to_wac130


function wax560_to_wac130() {
	global $CFG_update;

	// Wax Poteto 5.6.0 to Wacintaki Poteto 1.3.0
	web_line('Update from Wax Poteto 5.6.0 to Wacintaki 1.3.0...');

	/* Wax Poteto 5.6.0 now uses the same DB/config as Wac 1.3.x
		1) Verify that the database has a version market (5.6.0 oversight)

		2) Verify that the database is at version 1.3.0

		3) Update config file to say '1.3.0' instead of '5.6.0'

		4) Remove old Wax files
	*/


	include ('dbconn.php');
	include ('config.php');

	// Check DB version marker
	$result = mysql_query ("SELECT * FROM {$OekakiPoteto_Prefix}oekakimisc");
	if ($result) {
		if (mysql_num_rows ($result) != 3) {
			// Hmm... no miscstring!  Add it.
			$result = mysql_query ("ALTER TABLE {$OekakiPoteto_Prefix}oekakimisc ADD miscstring VARCHAR(14)");
			$result = mysql_query ("INSERT INTO {$OekakiPoteto_Prefix}oekakimisc VALUES (2, 'dbversion', 0, '1.3.0')");
		} else {
			// Make sure DB is 1.3.0
			$result = mysql_query ("SELECT miscstring FROM {$OekakiPoteto_Prefix}oekakimisc WHERE miscname='dbversion'");
			if (mysql_result ($result) != '1.3.0') {
				web_line('<strong>STOP</strong>: Wax Poteto database not at required version (1.3.0).  Run the Wax Poteto 5.6.x updater first.');
				mysql_close ($dbconn);
				return 0;
			}
		}
	} else {
		web_line('<strong>STOP</strong>: Could not verify Wax Poteto database version (miscstring)!  Try again.');
		mysql_close ($dbconn);
		return 0;
	}

	// Update config file
	$BBStitle = addslashes ($cfg['op_title']);

	$main_configuration = <<<EOF
<?php // Include only
	\$cfg = array();
	\$cfg['version'] = '1.3.0';

	\$cfg['op_title']  = '{$BBStitle}';
	\$cfg['op_author'] = '';
	\$cfg['op_adult']  = '{$cfg['op_adult']}';
	\$cfg['op_email']  = '{$cfg['op_email']}';
	\$cfg['op_url']    = '{$cfg['op_url']}';

	\$cfg['salt']    = '{$cfg['salt']}';
	\$cfg['op_pics'] = '{$cfg['op_pics']}';
	\$cfg['op_pre']  = '';

	\$cfg['kill_user']   = '{$cfg['kill_user']}';
	\$cfg['kill_reg']    = '{$cfg['kill_reg']}';
	\$cfg['approval']    = '{$cfg['approval']}';
	\$cfg['draw_immune'] = 'no';
	\$cfg['require_art'] = '{$cfg['require_art']}';
	\$cfg['guest_post']  = '{$cfg['guest_post']}';
	\$cfg['guest_draw']  = '{$cfg['guest_draw']}'; //WIP
	\$cfg['guest_adult'] = 'no';
	\$cfg['reg_draw']    = '{$cfg['reg_draw']}';
	\$cfg['reg_anim']    = '{$cfg['reg_anim']}';
	\$cfg['reg_upload']  = '{$cfg['reg_upload']}';
	\$cfg['reg_rules']   = 'no';

	\$cfg['language'] = 'english';
	\$cfg['template'] = '{$cfg['template']}';
	\$cfg['tpl_path'] = 'templates';
	\$cfg['res_path'] = 'resource';

	\$cfg['pic_store']  = '{$cfg['pic_store']}';
	\$cfg['pic_pages']  = '{$cfg['pic_pages']}';
	\$cfg['menu_pages'] = '{$cfg['menu_pages']}';
	\$cfg['pic_limit']      = '{$cfg['pic_limit']}'; //WIP
	\$cfg['pic_limit_time'] = '{$cfg['pic_limit_time']}'; //WIP

	\$cfg['porn_img']      = '{$cfg['porn_img']}';
	\$cfg['preview_img']   = '{$cfg['preview_img']}';
	\$cfg['preview_title'] = 'Preview';
	\$cfg['def_x']    = '{$cfg['def_x']}';
	\$cfg['def_y']    = '{$cfg['def_y']}';
	\$cfg['canvas_x'] = '{$cfg['canvas_x']}';
	\$cfg['canvas_y'] = '{$cfg['canvas_y']}';
	\$cfg['min_x']    = '{$cfg['min_x']}';
	\$cfg['min_y']    = '{$cfg['min_y']}';
	\$cfg['canvas_a'] = '{$cfg['canvas_a']}';
	\$cfg['max_anim'] = '{$cfg['max_anim']}';
	\$cfg['max_pic']  = '{$cfg['max_pic']}';

	\$cfg['use_thumb']   = '{$cfg['use_thumb']}';
	\$cfg['force_thumb'] = '{$cfg['force_thumb']}';
	\$cfg['thumb_bytes'] = '{$cfg['thumb_bytes']}';
	\$cfg['thumb_t']     = '{$cfg['thumb_t']}';
	\$cfg['thumb_r']     = '{$cfg['thumb_r']}';
	\$cfg['thumb_jpg']   = '{$cfg['thumb_jpg']}';

	\$cfg['use_mailbox'] = '{$cfg['use_mailbox']}';

	\$cfg['use_chat']   = '{$cfg['use_chat']}';
	\$cfg['chat_max']   = '{$cfg['chat_max']}';
	\$cfg['chat_pages'] = '{$cfg['chat_pages']}';

	\$cfg['smilies'] = '{$cfg['smilies']}';

	\$cfg['use_avatars']   = '{$cfg['use_avatars']}';
	\$cfg['use_c_avatars'] = '{$cfg['use_c_avatars']}';
	\$cfg['avatar_folder'] = '{$cfg['avatar_folder']}';
	\$cfg['avatar_x'] = '50';
	\$cfg['avatar_y'] = '50';

	\$cfg['public_retouch']   = 'yes';
	\$cfg['safety_storetime'] = '10';
	\$cfg['safety_saves']     = 'yes';
	\$cfg['self_bump']        = 'yes';
?>
EOF;
//<?

	// Write the config file
	if ($fp = fopen ('config.php', 'w')) {
		fwrite ($fp, $main_configuration);
		fclose ($fp);
	} else {
		web_line('<strong>STOP</strong>: Could not open config file "config.php" for writing.  Check your server permissions.');
		mysql_close ($dbconn);
		return 0;
	}

	// Remove files if they exist
	{
		@unlink('editbanner.php');
		@unlink('editnews.php');
		@unlink('ldelpics.php');
		@unlink('PCHViewer.jar');
		@unlink('tdesign.php');
		@unlink('tedit.php');
		@unlink('tedita.php');
		@unlink('res.zip');
		@unlink('tt.zip');
	}

	// Notice
	web_line('');
	web_line("<strong>NOTE</strong>: Remember to copy your resource files (banner.php, hosts.txt, ips.txt, pr0n.png, preview.png) into the Wacintaki 'resource' folder and CHMOD them so they are writable.");
	web_line('');

	// Complete!
	mysql_close ($dbconn);

	web_line('Update finished: Wacintaki Poteto 1.3.0');
	web_line('');
	return 1;
}
// END wax560_to_wac130


// -------------------------------------------------------------------
// Functions
// -------------------------------------------------------------------

// clean_images():
// returns TRUE, or string error
function clean_images($debug = FALSE) {
	global $dbconn, $cfg;
	global $OekakiPoteto_MemberPrefix, $OekakiPoteto_Prefix;
	global $CFG_update;

	$pre = $cfg['op_pre'];
	$extensions = array ('png', 'gif', 'jpg', 'pch', 'oeb', 'tmp');
	$files      = array(); // Filenames
	$good_files = array(); // Flags for good files (TRUE/FALSE)
	$deleted_files = 0;


	// Get list of files, by matching types
	$files = get_dir_list($cfg['op_pics'], $extensions);

	if (!$files) return 'No matching files';

	// Set all file flags to bad
	// NOTE: $file_count used later
	$file_count = count ($files);
	for ($i = 0; $i < $file_count; $i++) {
		$good_files[$i] = FALSE;
	}

	if ($debug) {
		web_line('Started clean of orphaned files...');
	}

	// Get each row from DB, and check what's present
	$result = mysql_query ("SELECT ID_2, PIC_ID, ptype, ttype, rtype, animation, datatype FROM {$OekakiPoteto_Prefix}oekakidta");
	if (!$result) return 'DB ('.mysql_error().')';

	$total_rows = mysql_num_rows ($result);

	if ($debug) {
		web_line($file_count.' total files.');
		web_line($total_rows.' total rows in db.');
	}

	for ($i = 0; $i < $total_rows; $i++) {
		$row = mysql_fetch_array ($result);

		// Build list of files to check
		$pic = array();
		if (!empty ($row['ptype']))
			$pic[] = $pre.$row['PIC_ID'].'.'.$row['ptype'];
		if (!empty ($row['ttype']))
			$pic[] = $pre.'t'.$row['PIC_ID'].'.'.$row['ttype'];
		if (!empty ($row['rtype']))
			$pic[] = $pre.'r'.$row['PIC_ID'].'.'.$row['rtype'];
		if ($row['animation']) {
			if ($row['datatype'] == 0 || $row['datatype'] == 2 || $row['datatype'] == 3) {
				$pic[] = $pre.$row['PIC_ID'].'.pch';
			}
			if ($row['datatype'] == 1) {
				$pic[] = $pre.$row['PIC_ID'].'.oeb';
			}
		}

		// Search for files
		foreach ($pic as $item) {
			// Each $pic
			for ($j = 0; $j < $file_count; $j++) {
				// Each file
				if (strpos ($files[$j], $item) !== FALSE) {
					// Found file!
					$good_files[$j] = TRUE;
					break;
				}
			}
		}
	}

	// Delete files without a good flag
	for ($i = 0; $i < $file_count; $i++) {
		if ($good_files[$i] == FALSE) {
			// Bad file
			if (!$debug) {
				@unlink ($cfg['op_pics'].'/'.$files[$i]);
				$deleted_files++;
			}
			if ($debug) {
				web_line($files[$i].': DELETE', TRUE);
			}
		}
		if ($debug) {
			web_line($files[$i]);
		}
	}

	if ($deleted_files > 0 || $debug) {
		web_line("Cleaned up {$deleted_files} orphaned files.");
	}

	if ($debug) {
		web_line('Finished cleaning orphaned files.');
	}

	return TRUE;
}

?>